// ########################################################################
// ## trigraphkatalog.cc:                                                ##
// ## Katalog von Listen zusammenhngender, trivalenter Graphen          ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ##           Arash Shahi,      shahi@xmailer.informatik.uni-bonn.de   ##
// ##           Jamshid Azizi,    azizi@xmailer.informatik.uni-bonn.de   ##
// ########################################################################


#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <bool.h>


#include "einstellungen.h"
#include "trigraphkatalog.h"


// ########################################################################
// ## Konstruktoren und Destruktor                                       ##
// ########################################################################
 
 
// Konstruktor 0
TriGraphKatalog::TriGraphKatalog()
{
   liste     = NULL;
   knoten    = 0;
   eintraege = 0;
   for(int i = 0;i < 3;i++)
      maxEintrag[i] = 0;
} 


// Klasse initialisieren
void TriGraphKatalog::Initialisiere(const unsigned int n)
{
   knoten        = n;
   eintraege     = 0;
   maxEintrag[2] = n + 1;
   maxEintrag[1] = n + 1;
   maxEintrag[0] = n + 1;  
   liste = new TriGraphListe***[maxEintrag[0]];
   for(unsigned int i = 0;i < maxEintrag[0];i++) {
      liste[i] = new TriGraphListe**[maxEintrag[1]];
      for(unsigned int j = 0;j < maxEintrag[1];j++) {
         liste[i][j] = new TriGraphListe*[maxEintrag[2]];
         for(unsigned int k = 0;k < maxEintrag[2];k++) {
            liste[i][j][k] = NULL;
         }
      }  
   }
}


// Klasse freigeben
void TriGraphKatalog::GibFrei()
{
   if(liste != NULL) {
      for(unsigned int i = 0;i < maxEintrag[0];i++) {
         for(unsigned int j = 0;j < maxEintrag[1];j++) {
            for(unsigned int k = 0;k < maxEintrag[2];k++) {
               delete liste[i][j][k];
               liste[i][j][k] = NULL;
            }
         }
      }                  
      liste  = NULL;
      knoten = 0;
      eintraege = 0;
      for(int i = 0;i < 3;i++) {
         maxEintrag[i] = 0;
      }          
   }                                                
}


// Clone-Operation
void TriGraphKatalog::Clone(const TriGraphKatalog& original)
{
   Initialisiere(original.knoten);
   eintraege = original.eintraege;
   for(unsigned int i = 0;i < maxEintrag[0];i++) {
      for(unsigned int j = 0;j < maxEintrag[1];j++) {
         for(unsigned int k = 0;k < maxEintrag[2];k++) {
            if(original.liste[i][j][k] != NULL) {
               liste[i][j][k] = new TriGraphListe(*original.liste[i][j][k]);            
            }
         }
      }
   }                        
}


// Konstruktor 1
TriGraphKatalog::TriGraphKatalog(const unsigned int n)
{
   Initialisiere(n);
} 


// Konstruktor 2
TriGraphKatalog::TriGraphKatalog(const TriGraphKatalog& original)
{
   Initialisiere(original.knoten);
   for(unsigned int i = 0;i < maxEintrag[0];i++) {
      for(unsigned int j = 0;j < maxEintrag[1];j++) {
         for(unsigned int k = 0;k < maxEintrag[2];k++) {
            liste[i][j][k] = original.liste[i][j][k];
         }
      }
   }                                                                  
} 


// Destruktor
TriGraphKatalog::~TriGraphKatalog()
{
   GibFrei();
} 


// ########################################################################
// ## Operatoren                                                         ##
// ########################################################################

 
// =-Operator
void TriGraphKatalog::operator=(const TriGraphKatalog& original)
{
   GibFrei();
   Clone(original);
}


// Output-Operator
ostream& operator<<(ostream& os, const TriGraphKatalog& katalog)
{
   if(katalog.liste != NULL) {
      os << "Katalogeintrge = " << katalog.eintraege << endl;
      for(unsigned int i = 0;i < katalog.maxEintrag[0];i++) {
         for(unsigned int j = 0;j < katalog.maxEintrag[1];j++) {
            for(unsigned int k = 0;k < katalog.maxEintrag[2];k++) {
               if(katalog.liste[i][j][k] != NULL) {
                  os << "===== Liste fr Gradpartition [";
                  os.width(3);
                  os << i << "][";
                  os.width(3);
                  os << j << "][";
                  os.width(3);
                  os << k << "] =================================" << endl;
                  os << *katalog.liste[i][j][k];
               }   
            }
         }
      }
   }            
   else {
      os << "{ leerer Katalog }" << endl;
   } 
   return(os);             
}


// ########################################################################
// ## Bearbeitung des Kataloges                                          ##
// ########################################################################


// Graph einfgen ohne bzgl. Isomorphien zu vergleichen
void TriGraphKatalog::FuegeEinOhneVergleich(const TriGraph& t)
{
#ifdef DEBUG
   if( (t.GradPartition(1) >= maxEintrag[0]) ||
       (t.GradPartition(2) >= maxEintrag[1]) ||
       (t.GradPartition(3) >= maxEintrag[2]) ) {
      cerr << "TriGraphKatalog::FuegeEinOhneVergleich - Unerwartete Gradpartition!" << endl;
      exit(1); 
   }
   if(knoten != t.Knoten()) {
      cerr << "TriGraphKatalog::FuegeEinOhneVergleich - Knotenzahl Graph <-> Katalog!" << endl;
      exit(1);
   }
#endif   

   TriGraphListe* l = 
      liste[t.GradPartition(1)][t.GradPartition(2)][t.GradPartition(3)];
   if(l == NULL) {
      l = new TriGraphListe();
      liste[t.GradPartition(1)][t.GradPartition(2)][t.GradPartition(3)] = l;
   }
   l->FuegeAnOhneVergleich(t);
   eintraege++;
}


// Graph einfgen
bool TriGraphKatalog::FuegeEin(const TriGraph& t)
{
#ifdef DEBUG
   if( (t.GradPartition(1) >= maxEintrag[0]) ||
       (t.GradPartition(2) >= maxEintrag[1]) ||
       (t.GradPartition(3) >= maxEintrag[2]) ) {
      cerr << "TriGraphKatalog::FuegeEin - Unerwartete Gradpartition!" << endl;
      exit(1); 
   }
   if(knoten != t.Knoten()) {
      cerr << "TriGraphKatalog::FuegeEin - Knotenzahl Graph <-> Katalog!" << endl;
      exit(1);
   }
#endif   

   TriGraphListe* l = 
      liste[t.GradPartition(1)][t.GradPartition(2)][t.GradPartition(3)];
   if(l == NULL) {
      l = new TriGraphListe();
      liste[t.GradPartition(1)][t.GradPartition(2)][t.GradPartition(3)] = l;
   }
   if(l->FuegeAn(t) == true) {
      eintraege++;
      return(true);
   }
   return(false);
}


// ########################################################################
// ## Lesen und Schreiben                                                ##
// ########################################################################


// Katalog aus Binrdatei lesen
bool TriGraphKatalog::Lese(istream& is)
{
   unsigned int alteKnotenzahl = knoten;
   GibFrei();
   Initialisiere(alteKnotenzahl);
   for(;;) {
      TriGraph graph;
      if(!graph.Lese(is)) {
         return(true);
      }
      FuegeEinOhneVergleich(graph);
   }
   return(true);
}


// Katalog in Binrdatei schreiben
bool TriGraphKatalog::Schreibe(ostream& os)
{
   if(liste != NULL) {
      for(unsigned int i = 0;i < maxEintrag[0];i++) {
         for(unsigned int j = 0;j < maxEintrag[1];j++) {
            for(unsigned int k = 0;k < maxEintrag[2];k++) {
               if(liste[i][j][k] != NULL) {
                  if(!liste[i][j][k]->Schreibe(os)) {
                     #ifdef DEBUG
                     cerr << "TriGraphKatalog::Schreibe() - Fehler beim Schreiben!" << endl;
                     #endif
                     return(false);
                  }
               }
            }
         }
      }
   }            
   return(true);
}
