// ########################################################################
// ## viewer.cc                                                          ##
// ## Viewer fr Listen zusammenhngender, trivalenter Graphen           ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <qapp.h>
#include <qfont.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qfiledlg.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qmsgbox.h>

#include "trigraph.h"
#include "trigraph.icc"
#include "trigraphliste.h"
#include "trigraphliste.icc"
int TriGraph::ausgabeEinstellungen = 0;


// Graph-Ausgabe-Klasse
class Graph : public QWidget
{
   Q_OBJECT
   public:
   Graph(QWidget* parent = NULL, const char* name = NULL);
   void drawIt() const;
   void paintEvent(QPaintEvent *);

   public slots:
   void ZeichneAlles(const unsigned int start,
                     QPainter* painter) const;
   void DruckeAlles(const unsigned int start,
                     QPainter* painter) const;
   void ZeichneGraph(QPainter* painter,
                     const unsigned int x, 
                     const unsigned int y,
                     const unsigned int breite,
                     const unsigned int hoehe,
                     TriGraph* t,
                     const unsigned int num) const;
   void ZeichneGraphen(const unsigned int start,
                       QPainter* painter,
                       const unsigned int wx,
                       const unsigned int wy,
                       const unsigned int wbreite,
                       const unsigned int whoehe) const;
};


// Viewer-Applikation
class Viewer : public QApplication
{
   Q_OBJECT
   public:
   Viewer(int argc, char** argv); 
   ~Viewer();
  
   public slots:
   void Plus();
   void Minus();
   void Ausgabe();
   void Neu();
   void Laden();
   void Hilfe();
   void MehrZeilen();
   void WenigerZeilen();
   void MehrSpalten();
   void WenigerSpalten();
   void ErsteSeite();
   void Format1x1();
   void Format4x2();
   void Format5x4();
   void Format9x7();
   void Format12x9();

   public:
   TriGraphListe* liste;
   int            start;
   unsigned int   anzahlX;
   unsigned int   anzahlY;
   Graph*         graph;
};

Viewer* gApp;


// Konstruktor fr Viewer-Applikation
Viewer::Viewer(int argc, char** argv)
   : QApplication(argc,argv)
{ 
   liste = new TriGraphListe;
   start = 0;
   anzahlX = 5;
   anzahlY = 4;
}


// Destruktor fr Viewer-Applikation
Viewer::~Viewer()
{
   delete liste;
}


// Nchste Seite anzeigen
void Viewer::Plus()
{
   int alt = start;
   start += anzahlX * anzahlY;
   if((unsigned int)start >= liste->Elemente()) {
      start = liste->Elemente() - anzahlX * anzahlY;
      if(start < 0)
         start = 0;
   }
   if(start < alt) {
      start = alt;  
   }
   if(start != alt) graph->drawIt();
}


// Vorherige Seite anzeigen
void Viewer::Minus()
{
   int alt = start;
   start -= anzahlX * anzahlY;
   if(start < 0)
      start = 0; 
   if(start != alt) graph->drawIt();
}


// PostScript-Ausgabe
void Viewer::Ausgabe()
{
   if(gApp->liste->Elemente() < 1) {
      QMessageBox::information(NULL,"TriGraph Viewer",
         "Es ist nicht sinnvoll, eine leere Liste zu drucken!",
         "&Abbruch",0);
      return;
   }

   QPrinter printer;
   printer.setOrientation(QPrinter::Landscape);
   printer.setCreator("TriGraph Viewer - Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de");
   printer.setDocName("Ausgabe von TriGraph Viewer");
   printer.setOutputFileName("ausgabe.ps");
   printer.setOutputToFile(FALSE);
   if(printer.setup(0)) {
      if(liste->Elemente() > 0) {
         unsigned int seite = 1;
         QPainter painter;
         painter.begin(&printer);
         painter.setWindow(0,0,842,595);
         painter.setPen(black);  
      
         for(unsigned int i = 0;i < liste->Elemente();i += anzahlX * anzahlY) {
            // berschrift drucken
            if(i > 1) {
               printer.newPage();
               seite++;
            }
            char str[128];
            painter.setFont(QFont("times",18,QFont::Bold));
            sprintf(str,"TriGraph Viewer -- Seite %d von %d",
                    seite,(int)ceil((double)liste->Elemente() / (double)(anzahlX * anzahlY)));
            painter.drawText(20,0,802,50,AlignLeft | AlignVCenter,str);
            painter.drawText(20,0,802,50,AlignRight | AlignVCenter,
                             "Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de");

            // Graphen drucken
            graph->DruckeAlles(i,&painter);
         }
         painter.end();
      }
   }
}


// Graphen-Liste leeren
void Viewer::Neu()
{
   start = 0;
   TriGraphListe null;
   *liste = null;
   graph->drawIt();
}


// Hilfe
void Viewer::Hilfe()
{
    QMessageBox::about(NULL,"TriGraph Viewer",
                       "Copyright Thomas Dreibholz\n\n"
                       "E-Mail: uzs6pq@ibm.rhrz.uni-bonn.de\n"
                       "WWW:    http://ibm.rhrz.uni-bonn.de/~uzs6pq");
}


// Seitenaufteilung ndern
void Viewer::MehrZeilen()
{
   if(anzahlY < 15) {
      anzahlY++;
      graph->drawIt();
   }
}


// Seitenaufteilung ndern
void Viewer::WenigerZeilen()
{
   if(anzahlY > 1) {
      anzahlY--;
      graph->drawIt();
   }
}


// Seitenaufteilung ndern
void Viewer::MehrSpalten()
{
   if(anzahlX < 25) {
      anzahlX++;
      graph->drawIt();
   }
}


// Seitenaufteilung ndern
void Viewer::WenigerSpalten()
{
   if(anzahlX > 1) {
      anzahlX--;
      graph->drawIt();
   }
}


// Zur erstem Seite
void Viewer::ErsteSeite()
{
   if(start > 0) {
      start = 0;
      graph->drawIt();
   }
}


// Format einstellen
void Viewer::Format1x1() { anzahlX = 1; anzahlY = 1; graph->drawIt(); }
void Viewer::Format4x2() { anzahlX = 4; anzahlY = 2; graph->drawIt(); }
void Viewer::Format5x4() { anzahlX = 5; anzahlY = 4; graph->drawIt(); }
void Viewer::Format9x7() { anzahlX = 9; anzahlY = 7; graph->drawIt(); }
void Viewer::Format12x9() { anzahlX = 12; anzahlY = 9; graph->drawIt(); }


// Graphen-Liste laden
void Viewer::Laden()
{
   QString name = QFileDialog::getOpenFileName();
   if(name.isNull())
      return;

   ifstream is(name,ios::binary | ios::in);
   if(is.good()) {   
      start = 0;
      liste->Lese(is);
      graph->drawIt();
   }
   else {
      QMessageBox::information(NULL,"TriGraph Viewer",
         "Fehler beim Einlesen der Graphen-Liste!",
         "&Abbruch",0);
   }
}


// Konstruktor fr Graph-Ausgabe
Graph::Graph(QWidget* parent, const char* name)
 : QWidget(parent,name)
{
    setMinimumSize(375,175);

    QPopupMenu *datei = new QPopupMenu();
    CHECK_PTR( datei );
    datei->insertItem("&Neu",   gApp,SLOT(Neu()),CTRL+Key_N);
    datei->insertItem("&ffnen",gApp,SLOT(Laden()),CTRL+Key_F);
    datei->insertSeparator();
    datei->insertItem("Liste &drucken...",gApp,SLOT(Ausgabe()),CTRL+Key_D);
    datei->insertSeparator();
    datei->insertItem("TriGraph Viewer b&eenden",qApp,SLOT(quit()),CTRL+Key_Q);

    QPopupMenu *strg = new QPopupMenu();
    CHECK_PTR( strg );
    strg->insertItem("Erste Seite",    gApp,SLOT(ErsteSeite()),CTRL+Key_W);
    strg->insertItem("Nchste Seite",  gApp,SLOT(Plus()),CTRL+Key_S);
    strg->insertItem("Vorherige Seite",gApp,SLOT(Minus()), CTRL+Key_A);

    QPopupMenu *einstellungen = new QPopupMenu();
    CHECK_PTR( einstellungen );
    einstellungen->insertItem("Mehr Zeilen",    gApp,SLOT(MehrZeilen()),CTRL+Key_Y);
    einstellungen->insertItem("Weniger Zeilen", gApp,SLOT(WenigerZeilen()),CTRL+Key_X);
    einstellungen->insertItem("Mehr Spalten",   gApp,SLOT(MehrSpalten()),CTRL+Key_C);
    einstellungen->insertItem("Weniger Spalten",gApp,SLOT(WenigerSpalten()),CTRL+Key_V);
    einstellungen->insertSeparator();
    einstellungen->insertItem("Vollbild",   gApp,SLOT(Format1x1()),CTRL+Key_E);
    einstellungen->insertItem("4x2-Format", gApp,SLOT(Format4x2()),CTRL+Key_R);
    einstellungen->insertItem("5x4-Format", gApp,SLOT(Format5x4()),CTRL+Key_T);
    einstellungen->insertItem("9x7-Format", gApp,SLOT(Format9x7()),CTRL+Key_Z);
    einstellungen->insertItem("12x9-Format",gApp,SLOT(Format12x9()),CTRL+Key_U);

    QPopupMenu *hilfe = new QPopupMenu();
    CHECK_PTR( hilfe );
    hilfe->insertItem("ber TriGraph Viewer...",gApp,SLOT(Hilfe()),CTRL+Key_H);

    QMenuBar* menu = new QMenuBar( this );
    CHECK_PTR( menu );
    menu->insertItem("&Datei",datei);
    menu->insertItem("&Steuerung",strg);
    menu->insertItem("&Einstellungen",einstellungen);
    menu->insertSeparator();
    menu->insertItem("&Hilfe",hilfe);
}


// Einzelnen Graphen zeichnen
void Graph::ZeichneGraph(QPainter* painter,
                         const unsigned int x, 
                         const unsigned int y,
                         const unsigned int breite,
                         const unsigned int hoehe,
                         TriGraph* t,
                         const unsigned int num) const
{
   painter->setPen(black);
   painter->drawRect(x,y,breite,hoehe);
   if(t != NULL) {
      const unsigned int mx = x + (breite / 2);
      const unsigned int my = y + (hoehe / 2);
      const double       hx = (double)breite / 2.8;
      const double       hy = (double)hoehe  / 2.8;
      const unsigned int N  = t->Knoten();
      double gp = 2.0 * M_PI / (double)N;
      unsigned int cx[N],cy[N];
      painter->setPen(black);
      for(unsigned int i = 0;i < N;i++) {
         cx[i] = mx + (int)(hx * cos((double)i * gp - M_PI_2));
         cy[i] = my + (int)(hy * sin((double)i * gp - M_PI_2));
         painter->drawEllipse(cx[i] - 2,cy[i] - 2,4,4);
      }
      painter->setPen(black);
      for(unsigned int i = 0;i < N;i++) {
         for(unsigned int j = i + 1;j < N;j++) {
            if(t->Verbunden(i,j)) {
               painter->drawLine(cx[i],cy[i],cx[j],cy[j]);  
            }
         }
      }
      char str[64];
      sprintf(str,"#%03d",num + 1);
      painter->setFont(QFont("times",10));
      painter->drawText(x + 2,y,breite - 4,14,AlignLeft|AlignTop|DontClip,str);
      sprintf(str,"%02d / %02d / %02d",
                  t->GradPartition(1),
                  t->GradPartition(2),
                  t->GradPartition(3));
      painter->setFont(QFont("times",8));
      painter->drawText(x,y + hoehe - 12,breite,12,
                        AlignHCenter|AlignBottom|DontClip,str);
   }
}


// Alle Graphen auf einer Seite zeichnen
void Graph::ZeichneGraphen(const unsigned int start,
                           QPainter* painter,
                           const unsigned int wx,
                           const unsigned int wy,
                           const unsigned int wbreite,
                           const unsigned int whoehe) const
{
   unsigned int breite = wbreite / gApp->anzahlX;
   unsigned int hoehe  = whoehe  / gApp->anzahlY;
   unsigned int num = 0;
   TriGraph graph;
  
   // Liste mit zum Startpunkt durchlaufen
   if(!gApp->liste->Erster(graph))
      return;     
   for(unsigned int i = 0;i < start;i++) {
      if(!gApp->liste->Nachfolger(graph))
         return;  
   }

   // Graphen zeichnen
   for(unsigned int y = 0;y < gApp->anzahlY;y++) {
      for(unsigned int x = 0;x < gApp->anzahlX;x++) {
         ZeichneGraph(painter,
                      wx + x * breite,
                      wy + y * hoehe,breite,hoehe,
                      &graph,start + num);
         num++;
         if(!gApp->liste->Nachfolger(graph))
            return;
      }
   }
}


// Seite zeichnen
void Graph::ZeichneAlles(const unsigned int start,
                         QPainter* painter) const
{
  char str[128];

  painter->eraseRect(0,0,width(),height());
  if(gApp->liste->Elemente() > 0) {
     sprintf(str,"Anzeige ab Graph %d von %d, insgesamt %d Seite(n)",
        gApp->start + 1, gApp->liste->Elemente(),
        (int)ceil((double)gApp->liste->Elemente() / (double)(gApp->anzahlX * gApp->anzahlY)));
  }
  else {
     strcpy(str,"Bitte eine Graphendatei ffnen!");
  } 
  painter->drawText(20,30,width() - 20,18,AlignLeft | AlignVCenter,str);
  ZeichneGraphen(start,painter,0,50,width(),height() - 50);
}


// Seite drucken
void Graph::DruckeAlles(const unsigned int start,
                        QPainter* painter) const
{
   ZeichneGraphen(start,painter,0,50,842,545);
}


// Seite zeichnen
void Graph::drawIt() const
{
   QPainter painter;
   painter.begin(this);
   ZeichneAlles(gApp->start,&painter);
   painter.end();
}


// paintEvent-Handler
void Graph::paintEvent(QPaintEvent *)
{
   QPainter painter;
   painter.begin(this);
   ZeichneAlles(gApp->start,&painter);
   painter.end();
}



#include "viewer.moc"

int main(int argc,char** argv)
{
   // Argumente auswerten
   Viewer a(argc,argv);
   gApp = &a;

   // Benutzeroberflche initialisieren
   a.graph = new Graph;
   a.graph->setGeometry(5,40,500,300);
   a.setMainWidget(a.graph);
   a.graph->show();

   // Programm starten
   a.Laden();
   return a.exec();
}
