// ########################################################################
// ## trigraph.h:                                                        ##
// ## Klasse fr zusammenhngende, trivalente Graphen                    ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ##           Arash Shahi,      shahi@xmailer.informatik.uni-bonn.de   ##
// ##           Jamshid Azizi,    azizi@xmailer.informatik.uni-bonn.de   ##
// ########################################################################

#ifndef TRIGRAPH_H
#define TRIGRAPH_H

#include <iostream.h>
#include <bool.h>

#include "einstellungen.h"


// Klasse fr trivalente Graphen
class TriGraph
{
   // ===== Konstruktoren und Destruktor ==================================
   public:
   TriGraph();
   TriGraph (const unsigned int n);
   TriGraph (const TriGraph& original);
   ~TriGraph();


   // ===== Zugriff auf Daten =============================================
   inline unsigned int Kanten() const;
   inline unsigned int Knoten() const;
   inline unsigned int GradPartition(const unsigned int grad) const;
   inline TriGraph*    Nachfolger() const;
   inline void         SetzeNachfolger(TriGraph* t);
     

   // ===== Operatoren ====================================================
   void              operator= (const TriGraph& original);
   friend inline int operator==(const TriGraph& t1, const TriGraph& t2);
   friend inline int operator!=(const TriGraph& t1, const TriGraph& t2);
   friend inline int operator< (const TriGraph& t1, const TriGraph& t2);
   friend inline int operator> (const TriGraph& t1, const TriGraph& t2);
   friend inline int operator<=(const TriGraph& t1, const TriGraph& t2);
   friend inline int operator>=(const TriGraph& t1, const TriGraph& t2);
   friend ostream&   operator<<(ostream& os, const TriGraph& graph);


   // ===== Voreinstellungen ==============================================
   static const int ausgabeVollstaendig = 0;
   static const int ausgabeKompakt      = (1 << 1);
   static const int ausgabeANSI         = (1 << 0);
   static int ausgabeEinstellungen;
   
    
   // ===== Verwaltung der Adjazenzmatrix =================================
   inline void Verbinde  (const unsigned int a, const unsigned int b);
   inline void Trenne    (const unsigned int a, const unsigned int b);
   inline bool Verbunden (const unsigned int a, const unsigned int b) const;
   inline bool Getrennt  (const unsigned int a, const unsigned int b) const;
   inline bool Verbindbar(const unsigned int a, const unsigned int b) const;
   inline unsigned int Grad(const unsigned int n) const;


   // ===== Laden und Speichern ===========================================
   bool Lese(istream& is);
   bool Schreibe(ostream& os);

   
   // ===== Interne Variablen =============================================
   private:
   unsigned short knoten;
   unsigned short kanten;
   unsigned short gradPartition[3];
   unsigned char* adjazenzMatrix;
   TriGraph*      nachfolger;       

    
   // ===== Interne Funktionen ============================================
   private:
   void Initialisiere(const unsigned int n);
   void GibFrei();
   void Clone(const TriGraph& original);    
   void SetzeKante(const unsigned int a, const unsigned int b,
                   const bool neueKante);
   void SetzeGradpartition();
};

// Speichermenge fr Adjazenzmatrix berechnen
inline unsigned int AdjazenzBytes(const unsigned int n);

#endif
